@@ -3,6 +3,7 @@
 
 #include <vector>
 #include <unordered_map>
+#include <map>
 
 enum attribute_op {
 	op_sum,
@@ -26,4 +27,6 @@ void set_attribute_accum(std::unordered_map<std::string, attribute_op> &attribut
 void set_attribute_accum(std::unordered_map<std::string, attribute_op> &attribute_accum, const char *arg, char **argv);
 void preserve_attribute(attribute_op const &op, const std::string &key, serial_val const &val, std::vector<std::string> &full_keys, std::vector<serial_val> &full_values, std::unordered_map<std::string, accum_state> &attribute_accum_state);
 
+extern std::map<std::string, attribute_op> numeric_operations;
+
 #endif
