@@ -22,8 +22,11 @@ bool preserve_input_order = false;
 std::unordered_map<std::string, attribute_op> attribute_accum;
 std::vector<std::string> unidecode_data;
 std::vector<mvt_layer> bins;
+std::string accumulate_numeric;
 
 std::set<std::string> keep;
+std::set<std::string> exclude;
+std::vector<std::string> exclude_prefix;
 
 void usage(char **argv) {
 	fprintf(stderr, "Usage: %s -o newtile.pbf.gz tile.pbf.gz oz/ox/oy nz/nx/ny\n", argv[0]);
@@ -40,11 +43,14 @@ int main(int argc, char **argv) {
 	double simplification = 0;
 	double tiny_polygon_size = 0;
 	std::string assign_to_bins;
+	std::string bin_by_id_list;
 
 	std::vector<input_tile> sources;
 
 	struct option long_options[] = {
 		{"include", required_argument, 0, 'y'},
+		{"exclude", required_argument, 0, 'x'},
+		{"exclude-prefix", required_argument, 0, 'x' & 0x1F},
 		{"full-detail", required_argument, 0, 'd'},
 		{"buffer", required_argument, 0, 'b'},
 		{"output", required_argument, 0, 'o'},
@@ -57,6 +63,8 @@ int main(int argc, char **argv) {
 		{"tiny-polygon-size", required_argument, 0, 's' & 0x1F},
 		{"source-tile", required_argument, 0, 't'},
 		{"assign-to-bins", required_argument, 0, 'b' & 0x1F},
+		{"bin-by-id-list", required_argument, 0, 'c' & 0x1F},
+		{"accumulate-numeric-attributes", required_argument, 0, 'a' & 0x1F},
 
 		{0, 0, 0, 0},
 	};
@@ -79,6 +87,14 @@ int main(int argc, char **argv) {
 			keep.insert(optarg);
 			break;
 
+		case 'x':
+			exclude.insert(optarg);
+			break;
+
+		case 'x' & 0x1F:
+			exclude_prefix.push_back(optarg);
+			break;
+
 		case 'o':
 			outfile = optarg;
 			break;
@@ -127,6 +143,14 @@ int main(int argc, char **argv) {
 			assign_to_bins = optarg;
 			break;
 
+		case 'c' & 0x1F:
+			bin_by_id_list = optarg;
+			break;
+
+		case 'a' & 0x1F:
+			accumulate_numeric = optarg;
+			break;
+
 		default:
 			fprintf(stderr, "Unrecognized flag -%c\n", i);
 			usage(argv);
@@ -197,7 +221,7 @@ int main(int argc, char **argv) {
 			exit(EXIT_OPEN);
 		}
 
-		bins = parse_layers(f, nz, nx, ny, 1LL << detail);
+		bins = parse_layers(f, nz, nx, ny, 1LL << detail, true);
 		fclose(f);
 	}
 
@@ -227,7 +251,7 @@ int main(int argc, char **argv) {
 		its.push_back(std::move(t));
 	}
 
-	std::string out = overzoom(its, nz, nx, ny, detail, buffer, keep, true, NULL, demultiply, json_filter, preserve_input_order, attribute_accum, unidecode_data, simplification, tiny_polygon_size, bins);
+	std::string out = overzoom(its, nz, nx, ny, detail, buffer, keep, exclude, exclude_prefix, true, NULL, demultiply, json_filter, preserve_input_order, attribute_accum, unidecode_data, simplification, tiny_polygon_size, bins, bin_by_id_list, accumulate_numeric);
 
 	FILE *f = fopen(outfile, "wb");
 	if (f == NULL) {
